diff mbox

[4/4] KVM: Disable SMAP for guests in EPT realmode and EPT unpaging mode

Message ID 1395923135-15329-5-git-send-email-feng.wu@intel.com (mailing list archive)
State New, archived
Headers show

Commit Message

Wu, Feng March 27, 2014, 12:25 p.m. UTC
SMAP is disabled if CPU is in non-paging mode in hardware.
However KVM always uses paging mode to emulate guest non-paging
mode with TDP. To emulate this behavior, SMAP needs to be
manually disabled when guest switches to non-paging mode.

Signed-off-by: Feng Wu <feng.wu@intel.com>
---
 arch/x86/kvm/vmx.c | 10 ++++++----
 1 file changed, 6 insertions(+), 4 deletions(-)

Comments

Jan Kiszka March 27, 2014, 4:14 p.m. UTC | #1
On 2014-03-27 13:25, Feng Wu wrote:
> SMAP is disabled if CPU is in non-paging mode in hardware.
> However KVM always uses paging mode to emulate guest non-paging
> mode with TDP. To emulate this behavior, SMAP needs to be
> manually disabled when guest switches to non-paging mode.
> 
> Signed-off-by: Feng Wu <feng.wu@intel.com>
> ---
>  arch/x86/kvm/vmx.c | 10 ++++++----
>  1 file changed, 6 insertions(+), 4 deletions(-)
> 
> diff --git a/arch/x86/kvm/vmx.c b/arch/x86/kvm/vmx.c
> index dcc4de3..1d37e50 100644
> --- a/arch/x86/kvm/vmx.c
> +++ b/arch/x86/kvm/vmx.c
> @@ -3421,13 +3421,15 @@ static int vmx_set_cr4(struct kvm_vcpu *vcpu, unsigned long cr4)
>  			hw_cr4 &= ~X86_CR4_PAE;
>  			hw_cr4 |= X86_CR4_PSE;
>  			/*
> -			 * SMEP is disabled if CPU is in non-paging mode in
> -			 * hardware. However KVM always uses paging mode to
> +			 * SMEP/SMAP is disabled if CPU is in non-paging mode
> +			 * in hardware. However KVM always uses paging mode to
>  			 * emulate guest non-paging mode with TDP.
> -			 * To emulate this behavior, SMEP needs to be manually
> -			 * disabled when guest switches to non-paging mode.
> +			 * To emulate this behavior, SMEP/SMAP needs to be
> +			 * manually disabled when guest switches to non-paging
> +			 * mode.
>  			 */
>  			hw_cr4 &= ~X86_CR4_SMEP;
> +			hw_cr4 &= ~X86_CR4_SMAP;

Why not

hw_cr4 &= ~(X86_CR4_SMEP | X86_CR4_SMAP);

?

Jan

>  		} else if (!(cr4 & X86_CR4_PAE)) {
>  			hw_cr4 &= ~X86_CR4_PAE;
>  		}
>
Wu, Feng March 28, 2014, 12:41 a.m. UTC | #2
> -----Original Message-----
> From: kvm-owner@vger.kernel.org [mailto:kvm-owner@vger.kernel.org] On
> Behalf Of Jan Kiszka
> Sent: Friday, March 28, 2014 12:15 AM
> To: Wu, Feng; pbonzini@redhat.com; gleb@redhat.com; hpa@zytor.com;
> kvm@vger.kernel.org
> Subject: Re: [PATCH 4/4] KVM: Disable SMAP for guests in EPT realmode and
> EPT unpaging mode
> 
> On 2014-03-27 13:25, Feng Wu wrote:
> > SMAP is disabled if CPU is in non-paging mode in hardware.
> > However KVM always uses paging mode to emulate guest non-paging
> > mode with TDP. To emulate this behavior, SMAP needs to be
> > manually disabled when guest switches to non-paging mode.
> >
> > Signed-off-by: Feng Wu <feng.wu@intel.com>
> > ---
> >  arch/x86/kvm/vmx.c | 10 ++++++----
> >  1 file changed, 6 insertions(+), 4 deletions(-)
> >
> > diff --git a/arch/x86/kvm/vmx.c b/arch/x86/kvm/vmx.c
> > index dcc4de3..1d37e50 100644
> > --- a/arch/x86/kvm/vmx.c
> > +++ b/arch/x86/kvm/vmx.c
> > @@ -3421,13 +3421,15 @@ static int vmx_set_cr4(struct kvm_vcpu *vcpu,
> unsigned long cr4)
> >  			hw_cr4 &= ~X86_CR4_PAE;
> >  			hw_cr4 |= X86_CR4_PSE;
> >  			/*
> > -			 * SMEP is disabled if CPU is in non-paging mode in
> > -			 * hardware. However KVM always uses paging mode to
> > +			 * SMEP/SMAP is disabled if CPU is in non-paging mode
> > +			 * in hardware. However KVM always uses paging mode to
> >  			 * emulate guest non-paging mode with TDP.
> > -			 * To emulate this behavior, SMEP needs to be manually
> > -			 * disabled when guest switches to non-paging mode.
> > +			 * To emulate this behavior, SMEP/SMAP needs to be
> > +			 * manually disabled when guest switches to non-paging
> > +			 * mode.
> >  			 */
> >  			hw_cr4 &= ~X86_CR4_SMEP;
> > +			hw_cr4 &= ~X86_CR4_SMAP;
> 
> Why not
> 
> hw_cr4 &= ~(X86_CR4_SMEP | X86_CR4_SMAP);
> 
> ?

Sure, your suggestion is cleaner, I will change it in the next version, thanks for the comments!

> 
> Jan
> 
> >  		} else if (!(cr4 & X86_CR4_PAE)) {
> >  			hw_cr4 &= ~X86_CR4_PAE;
> >  		}
> >
> 

Thanks,
Feng
--
To unsubscribe from this list: send the line "unsubscribe kvm" in
the body of a message to majordomo@vger.kernel.org
More majordomo info at  http://vger.kernel.org/majordomo-info.html
diff mbox

Patch

diff --git a/arch/x86/kvm/vmx.c b/arch/x86/kvm/vmx.c
index dcc4de3..1d37e50 100644
--- a/arch/x86/kvm/vmx.c
+++ b/arch/x86/kvm/vmx.c
@@ -3421,13 +3421,15 @@  static int vmx_set_cr4(struct kvm_vcpu *vcpu, unsigned long cr4)
 			hw_cr4 &= ~X86_CR4_PAE;
 			hw_cr4 |= X86_CR4_PSE;
 			/*
 			 * SMEP is disabled if CPU is in non-paging mode in
 			 * hardware. However KVM always uses paging mode to
 			 * emulate guest non-paging mode with TDP.
 			 * To emulate this behavior, SMEP needs to be manually
 			 * disabled when guest switches to non-paging mode.
 			 */
 			hw_cr4 &= ~X86_CR4_SMEP;
+			hw_cr4 &= ~X86_CR4_SMAP;
 		} else if (!(cr4 & X86_CR4_PAE)) {
 			hw_cr4 &= ~X86_CR4_PAE;
 		}
@@ -3421,13 +3421,15 @@  static int vmx_set_cr4(struct kvm_vcpu *vcpu, unsigned long cr4)
 			hw_cr4 |= X86_CR4_PSE;
 			/*
-			 * SMEP is disabled if CPU is in non-paging mode in
-			 * hardware. However KVM always uses paging mode to
+			 * SMEP/SMAP is disabled if CPU is in non-paging mode
+			 * in hardware. However KVM always uses paging mode to
 			 * emulate guest non-paging mode with TDP.
@@ -3421,13 +3421,15 @@  static int vmx_set_cr4(struct kvm_vcpu *vcpu, unsigned long cr4)
 			 * emulate guest non-paging mode with TDP.
-			 * To emulate this behavior, SMEP needs to be manually
-			 * disabled when guest switches to non-paging mode.
+			 * To emulate this behavior, SMEP/SMAP needs to be
+			 * manually disabled when guest switches to non-paging
+			 * mode.
 			 */
 			hw_cr4 &= ~X86_CR4_SMEP;